@@ -30,12 +30,14 @@
 
 #include "gstrtponviftimestamp.h"
 
-#define GST_NTP_OFFSET_EVENT_NAME "GstNtpOffset"
+#define GST_ONVIF_TIMESTAMP_EVENT_NAME "GstOnvifTimestamp"
 
 #define DEFAULT_NTP_OFFSET GST_CLOCK_TIME_NONE
 #define DEFAULT_CSEQ 0
 #define DEFAULT_SET_E_BIT FALSE
 #define DEFAULT_SET_T_BIT FALSE
+#define DEFAULT_DROP_OUT_OF_SEGMENT TRUE
+#define DEFAULT_USE_REFERENCE_TIMESTAMPS FALSE
 
 GST_DEBUG_CATEGORY_STATIC (rtponviftimestamp_debug);
 #define GST_CAT_DEFAULT (rtponviftimestamp_debug)
@@ -71,11 +73,15 @@ enum
   PROP_CSEQ,
   PROP_SET_E_BIT,
   PROP_SET_T_BIT,
+  PROP_DROP_OUT_OF_SEGMENT,
+  PROP_USE_REFERENCE_TIMESTAMPS
 };
 
 /*static guint gst_rtp_onvif_timestamp_signals[LAST_SIGNAL] = { 0 }; */
 
 G_DEFINE_TYPE (GstRtpOnvifTimestamp, gst_rtp_onvif_timestamp, GST_TYPE_ELEMENT);
+GST_ELEMENT_REGISTER_DEFINE (rtponviftimestamp, "rtponviftimestamp",
+    GST_RANK_NONE, GST_TYPE_RTP_ONVIF_TIMESTAMP);
 
 static void
 gst_rtp_onvif_timestamp_get_property (GObject * object,
@@ -96,6 +102,12 @@ gst_rtp_onvif_timestamp_get_property (GObject * object,
     case PROP_SET_T_BIT:
       g_value_set_boolean (value, self->prop_set_t_bit);
       break;
+    case PROP_DROP_OUT_OF_SEGMENT:
+      g_value_set_boolean (value, self->prop_drop_out_of_segment);
+      break;
+    case PROP_USE_REFERENCE_TIMESTAMPS:
+      g_value_set_boolean (value, self->prop_use_reference_timestamps);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -121,6 +133,12 @@ gst_rtp_onvif_timestamp_set_property (GObject * object,
     case PROP_SET_T_BIT:
       self->prop_set_t_bit = g_value_get_boolean (value);
       break;
+    case PROP_DROP_OUT_OF_SEGMENT:
+      self->prop_drop_out_of_segment = g_value_get_boolean (value);
+      break;
+    case PROP_USE_REFERENCE_TIMESTAMPS:
+      self->prop_use_reference_timestamps = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -194,9 +212,18 @@ gst_rtp_onvif_timestamp_change_state (GstElement * element,
 
   switch (transition) {
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      self->ntp_offset = self->prop_ntp_offset;
-      GST_DEBUG_OBJECT (self, "ntp-offset: %" GST_TIME_FORMAT,
-          GST_TIME_ARGS (self->ntp_offset));
+      if (self->prop_use_reference_timestamps &&
+          self->prop_ntp_offset != DEFAULT_NTP_OFFSET) {
+        GST_WARNING_OBJECT (self, "ntp-offset should not be set if reference "
+            "timestamps are used");
+        self->ntp_offset = DEFAULT_NTP_OFFSET;
+      } else if (self->prop_use_reference_timestamps) {
+        GST_DEBUG_OBJECT (self, "using reference timestamp meta");
+      } else {
+        self->ntp_offset = self->prop_ntp_offset;
+        GST_DEBUG_OBJECT (self, "ntp-offset: %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (self->ntp_offset));
+      }
       self->set_d_bit = TRUE;
       self->set_e_bit = FALSE;
       self->set_t_bit = FALSE;
@@ -229,6 +256,7 @@ gst_rtp_onvif_timestamp_finalize (GObject * object)
   GstRtpOnvifTimestamp *self = GST_RTP_ONVIF_TIMESTAMP (object);
 
   g_queue_free (self->event_queue);
+  gst_caps_replace (&self->reference_timestamp_id, NULL);
 
   G_OBJECT_CLASS (gst_rtp_onvif_timestamp_parent_class)->finalize (object);
 }
@@ -271,6 +299,35 @@ gst_rtp_onvif_timestamp_class_init (GstRtpOnvifTimestampClass * klass)
           "extension. This increases latency by one packet",
           DEFAULT_SET_T_BIT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (gobject_class, PROP_DROP_OUT_OF_SEGMENT,
+      g_param_spec_boolean ("drop-out-of-segment", "Drop out of segment",
+          "Whether the element should drop buffers that fall outside the segment, "
+          "not part of the specification but allows full reverse playback.",
+          DEFAULT_DROP_OUT_OF_SEGMENT,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstRtpOnvifTimestamp:use-reference-timestamps:
+   *
+   * Whether to obtain timestamps from reference timestamp meta instead of using
+   * the ntp-offset method. If enabled then timestamps are expected to be
+   * attached to the buffers, and in that case ntp-offset should not be
+   * configured.
+   *
+   * Default value is FALSE, meaning that the ntp-offset property is used.
+   * If neither is set then the element calculates an ntp-offset.
+   *
+   * Since: 1.22
+   */
+  g_object_class_install_property (gobject_class, PROP_USE_REFERENCE_TIMESTAMPS,
+      g_param_spec_boolean ("use-reference-timestamps",
+          "Use reference timestamps",
+          "Whether the element should use reference UTC timestamps from the "
+          "buffers instead of using the ntp-offset mechanism.",
+          DEFAULT_USE_REFERENCE_TIMESTAMPS,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS |
+          GST_PARAM_MUTABLE_READY));
+
   /* register pads */
   gst_element_class_add_static_pad_template (gstelement_class,
       &sink_template_factory);
@@ -330,9 +387,9 @@ gst_rtp_onvif_timestamp_sink_event (GstPad * pad, GstObject * parent,
     case GST_EVENT_CUSTOM_DOWNSTREAM:
       /* if the "set-e-bit" property is set, an offset event might mark the
        * stream as discontinued. We need to check if the currently cached buffer
-       * needs the e-bit before it's pushed */
-      if (self->buffer != NULL && self->prop_set_e_bit &&
-          gst_event_has_name (event, GST_NTP_OFFSET_EVENT_NAME)) {
+       * or buffer list needs the e-bit before it's pushed */
+      if ((self->buffer != NULL || self->list != NULL) && self->prop_set_e_bit
+          && gst_event_has_name (event, GST_ONVIF_TIMESTAMP_EVENT_NAME)) {
         gboolean discont;
         if (parse_event_ntp_offset (self, event, NULL, &discont)) {
           GST_DEBUG_OBJECT (self, "stream %s discontinued",
@@ -374,7 +431,7 @@ gst_rtp_onvif_timestamp_sink_event (GstPad * pad, GstObject * parent,
 
   /* enqueue serialized events if there is a cached buffer */
   if (GST_EVENT_IS_SERIALIZED (event) && (self->buffer || self->list)) {
-    GST_DEBUG ("enqueueing serialized event");
+    GST_WARNING ("enqueueing serialized event");
     g_queue_push_tail (self->event_queue, event);
     event = NULL;
     goto out;
@@ -386,7 +443,7 @@ gst_rtp_onvif_timestamp_sink_event (GstPad * pad, GstObject * parent,
       /* update the ntp-offset after any cached buffer/buffer list has been
        * pushed. the d-bit of the next buffer/buffer list should be set if
        * the stream is discontinued */
-      if (gst_event_has_name (event, GST_NTP_OFFSET_EVENT_NAME)) {
+      if (gst_event_has_name (event, GST_ONVIF_TIMESTAMP_EVENT_NAME)) {
         GstClockTime offset;
         gboolean discont;
         if (parse_event_ntp_offset (self, event, &offset, &discont)) {
@@ -435,9 +492,13 @@ gst_rtp_onvif_timestamp_init (GstRtpOnvifTimestamp * self)
       gst_pad_new_from_static_template (&src_template_factory, "src");
   gst_element_add_pad (GST_ELEMENT (self), self->srcpad);
 
+  self->prop_use_reference_timestamps = DEFAULT_USE_REFERENCE_TIMESTAMPS;
+  self->reference_timestamp_id = gst_caps_new_empty_simple ("timestamp/x-unix");
+
   self->prop_ntp_offset = DEFAULT_NTP_OFFSET;
   self->prop_set_e_bit = DEFAULT_SET_E_BIT;
   self->prop_set_t_bit = DEFAULT_SET_T_BIT;
+  self->prop_drop_out_of_segment = DEFAULT_DROP_OUT_OF_SEGMENT;
 
   gst_segment_init (&self->segment, GST_FORMAT_UNDEFINED);
 
@@ -449,6 +510,51 @@ gst_rtp_onvif_timestamp_init (GstRtpOnvifTimestamp * self)
 #define EXTENSION_ID 0xABAC
 #define EXTENSION_SIZE 3
 
+static guint64
+get_utc_from_reference_timestamp (GstRtpOnvifTimestamp * self, GstBuffer * buf)
+{
+  GstReferenceTimestampMeta *meta;
+  GstClockTime time;
+
+  meta = gst_buffer_get_reference_timestamp_meta (buf,
+      self->reference_timestamp_id);
+  if (meta != NULL) {
+    /* the reference timestamp is expressed in unix times so add the difference
+     * between unix and ntp epochs */
+    time = meta->timestamp + G_GUINT64_CONSTANT (2208988800) * GST_SECOND;
+    GST_TRACE_OBJECT (self, "UTC reference timestamp found: %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (time));
+  } else {
+    GST_ERROR_OBJECT (self, "UTC reference timestamp not found");
+    time = GST_CLOCK_TIME_NONE;
+  }
+
+  return time;
+}
+
+static guint64
+get_utc_from_offset (GstRtpOnvifTimestamp * self, GstBuffer * buf)
+{
+  guint64 time = GST_CLOCK_TIME_NONE;
+
+  if (GST_BUFFER_PTS_IS_VALID (buf)) {
+    time = gst_segment_to_stream_time (&self->segment, GST_FORMAT_TIME,
+        GST_BUFFER_PTS (buf));
+  } else if (GST_BUFFER_DTS_IS_VALID (buf)) {
+    time = gst_segment_to_stream_time (&self->segment, GST_FORMAT_TIME,
+        GST_BUFFER_DTS (buf));
+  } else {
+    g_assert_not_reached ();
+  }
+
+  /* add the offset (in seconds) */
+  if (time != GST_CLOCK_TIME_NONE) {
+    time += self->ntp_offset;
+  }
+
+  return time;
+}
+
 static gboolean
 handle_buffer (GstRtpOnvifTimestamp * self, GstBuffer * buf)
 {
@@ -459,7 +565,8 @@ handle_buffer (GstRtpOnvifTimestamp * self, GstBuffer * buf)
   guint64 time;
   guint8 field = 0;
 
-  if (!GST_CLOCK_TIME_IS_VALID (self->ntp_offset)) {
+  if (!self->prop_use_reference_timestamps &&
+      !GST_CLOCK_TIME_IS_VALID (self->ntp_offset)) {
     GstClock *clock = gst_element_get_clock (GST_ELEMENT (self));
 
     if (clock) {
@@ -517,13 +624,19 @@ handle_buffer (GstRtpOnvifTimestamp * self, GstBuffer * buf)
     return FALSE;
   }
 
-  /* NTP timestamp */
-  if (GST_BUFFER_PTS_IS_VALID (buf)) {
-    time = gst_segment_to_stream_time (&self->segment, GST_FORMAT_TIME,
-        GST_BUFFER_PTS (buf));
-  } else if (GST_BUFFER_DTS_IS_VALID (buf)) {
-    time = gst_segment_to_stream_time (&self->segment, GST_FORMAT_TIME,
-        GST_BUFFER_DTS (buf));
+  if (self->prop_use_reference_timestamps) {
+    time = get_utc_from_reference_timestamp (self, buf);
+    if (time == GST_CLOCK_TIME_NONE) {
+      gst_rtp_buffer_unmap (&rtp);
+      return FALSE;
+    }
+  } else if (GST_BUFFER_PTS_IS_VALID (buf) || GST_BUFFER_DTS_IS_VALID (buf)) {
+    time = get_utc_from_offset (self, buf);
+    if (self->prop_drop_out_of_segment && time == GST_CLOCK_TIME_NONE) {
+      GST_ERROR_OBJECT (self, "Failed to get stream time");
+      gst_rtp_buffer_unmap (&rtp);
+      return FALSE;
+    }
   } else {
     GST_INFO_OBJECT (self,
         "Buffer doesn't contain any valid DTS or PTS timestamp");
@@ -531,13 +644,11 @@ handle_buffer (GstRtpOnvifTimestamp * self, GstBuffer * buf)
   }
 
   if (time == GST_CLOCK_TIME_NONE) {
-    GST_ERROR_OBJECT (self, "Failed to get stream time");
-    goto done;
+    GST_ERROR_OBJECT (self, "failed calculating timestamp");
+    gst_rtp_buffer_unmap (&rtp);
+    return FALSE;
   }
 
-  /* add the offset (in seconds) */
-  time += self->ntp_offset;
-
   /* convert to NTP time. upper 32 bits should contain the seconds
    * and the lower 32 bits, the fractions of a second. */
   time = gst_util_uint64_scale (time, (G_GINT64_CONSTANT (1) << 32),
@@ -556,7 +667,7 @@ handle_buffer (GstRtpOnvifTimestamp * self, GstBuffer * buf)
     field |= (1 << 7);
   }
 
-  /* Set E if the next buffer has DISCONT */
+  /* Set E if this the last buffer of a contiguous section of recording */
   if (self->set_e_bit) {
     GST_DEBUG_OBJECT (self, "set E flag");
     field |= (1 << 6);
@@ -564,7 +675,7 @@ handle_buffer (GstRtpOnvifTimestamp * self, GstBuffer * buf)
   }
 
   /* Set D if the buffer has the DISCONT flag */
-  if (self->set_d_bit) {
+  if (self->set_d_bit || GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_DISCONT)) {
     GST_DEBUG_OBJECT (self, "set D flag");
     field |= (1 << 5);
     self->set_d_bit = FALSE;
@@ -621,15 +732,18 @@ gst_rtp_onvif_timestamp_chain (GstPad * pad, GstObject * parent,
   return result;
 }
 
+static gboolean
+do_handle_buffer (GstBuffer ** buffer, guint idx, GstRtpOnvifTimestamp * self)
+{
+  return handle_buffer (self, *buffer);
+}
+
 /* @buf: (transfer full) */
 static GstFlowReturn
 handle_and_push_buffer_list (GstRtpOnvifTimestamp * self, GstBufferList * list)
 {
-  GstBuffer *buf;
-
-  /* Set the extension on the *first* buffer */
-  buf = gst_buffer_list_get (list, 0);
-  if (!handle_buffer (self, buf)) {
+  if (!gst_buffer_list_foreach (list, (GstBufferListFunc) do_handle_buffer,
+          self)) {
     gst_buffer_list_unref (list);
     return GST_FLOW_ERROR;
   }
