@@ -232,6 +232,7 @@ gst_nv_h265_encoder_check_reconfigure (GstNvEncoder * encoder,
 static gboolean gst_nv_h265_encoder_select_device (GstNvEncoder * encoder,
     const GstVideoInfo * info, GstBuffer * buffer,
     GstNvEncoderDeviceData * data);
+static guint gst_nv_h265_encoder_calculate_min_buffers (GstNvEncoder * encoder);
 
 static void
 gst_nv_h265_encoder_class_init (GstNvH265EncoderClass * klass, gpointer data)
@@ -468,6 +469,8 @@ gst_nv_h265_encoder_class_init (GstNvH265EncoderClass * klass, gpointer data)
       GST_DEBUG_FUNCPTR (gst_nv_h265_encoder_check_reconfigure);
   nvenc_class->select_device =
       GST_DEBUG_FUNCPTR (gst_nv_h265_encoder_select_device);
+  nvenc_class->calculate_min_buffers =
+      GST_DEBUG_FUNCPTR (gst_nv_h265_encoder_calculate_min_buffers);
 
   klass->device_caps = cdata->device_caps;
   klass->cuda_device_id = cdata->cuda_device_id;
@@ -1252,7 +1255,7 @@ gst_nv_h265_encoder_set_format (GstNvEncoder * encoder,
       rc_params->constQP.qpIntra = self->qp_i;
     if (self->qp_p >= 0)
       rc_params->constQP.qpInterP = self->qp_p;
-    if (self->qp_p >= 0)
+    if (self->qp_b >= 0)
       rc_params->constQP.qpInterB = self->qp_b;
   }
 
@@ -1750,6 +1753,24 @@ gst_nv_h265_encoder_select_device (GstNvEncoder * encoder,
   return TRUE;
 }
 
+static guint
+gst_nv_h265_encoder_calculate_min_buffers (GstNvEncoder * encoder)
+{
+  GstNvH265Encoder *self = GST_NV_H265_ENCODER (encoder);
+  guint num_buffers;
+
+  /* At least 4 surfaces are required as documented by Nvidia Encoder guide */
+  num_buffers = 4;
+
+  /* lookahead depth */
+  num_buffers += self->rc_lookahead;
+
+  /* B frames + 1 */
+  num_buffers += self->bframes + 1;
+
+  return num_buffers;
+}
+
 static GstNvEncoderClassData *
 gst_nv_h265_encoder_create_class_data (GstObject * device, gpointer session,
     GstNvEncoderDeviceMode device_mode)
@@ -2097,9 +2118,9 @@ gst_nv_h265_encoder_register_auto_select (GstPlugin * plugin,
   std::string resolution_str;
   GList *iter;
   guint adapter_luid_size = 0;
-  gint64 adapter_luid_list[8];
+  gint64 adapter_luid_list[8] = { 0, };
   guint cuda_device_id_size = 0;
-  guint cuda_device_id_list[8];
+  guint cuda_device_id_list[8] = { 0, };
   GstNvEncoderDeviceCaps dev_caps;
   GstNvEncoderClassData *cdata;
   GstCaps *sink_caps = nullptr;
